@@ -215,6 +215,8 @@ class GameNode(abc.ABC):
         ``Variant``) unless the ``FEN`` header tag is set.
 
         It's a copy, so modifying the board will not alter the game.
+
+        Complexity is `O(n)`.
         """
 
     @abc.abstractmethod
@@ -226,11 +228,15 @@ class GameNode(abc.ABC):
 
         Usually this is equal to the number of parent nodes, but it may be
         more if the game was started from a custom position.
+
+        Complexity is `O(n)`.
         """
 
     def turn(self) -> Color:
         """
         Gets the color to move at this node. See :data:`chess.Board.turn`.
+
+        Complexity is `O(n)`.
         """
         return self.ply() % 2 == 0
 
@@ -241,13 +247,21 @@ class GameNode(abc.ABC):
         return node
 
     def game(self) -> Game:
-        """Gets the root node, i.e., the game."""
+        """
+        Gets the root node, i.e., the game.
+
+        Complexity is `O(n)`.
+        """
         root = self.root()
         assert isinstance(root, Game), "GameNode not rooted in Game"
         return root
 
     def end(self) -> GameNode:
-        """Follows the main variation to the end and returns the last node."""
+        """
+        Follows the main variation to the end and returns the last node.
+
+        Complexity is `O(n)`.
+        """
         node = self
 
         while node.variations:
@@ -256,7 +270,11 @@ class GameNode(abc.ABC):
         return node
 
     def is_end(self) -> bool:
-        """Checks if this node is the last node in the current variation."""
+        """
+        Checks if this node is the last node in the current variation.
+
+        Complexity is `O(1)`.
+        """
         return not self.variations
 
     def starts_variation(self) -> bool:
@@ -267,6 +285,8 @@ class GameNode(abc.ABC):
 
         For example, in ``1. e4 e5 (1... c5 2. Nf3) 2. Nf3``, the node holding
         1... c5 starts a variation.
+
+        Complexity is `O(1)`.
         """
         if not self.parent or not self.parent.variations:
             return False
@@ -274,7 +294,11 @@ class GameNode(abc.ABC):
         return self.parent.variations[0] != self
 
     def is_mainline(self) -> bool:
-        """Checks if the node is in the mainline of the game."""
+        """
+        Checks if the node is in the mainline of the game.
+
+        Complexity is `O(n)`.
+        """
         node = self
 
         while node.parent:
@@ -291,6 +315,8 @@ class GameNode(abc.ABC):
         """
         Checks if this node is the first variation from the point of view of its
         parent. The root node is also in the main variation.
+
+        Complexity is `O(1)`.
         """
         if not self.parent:
             return True
@@ -367,6 +393,8 @@ class GameNode(abc.ABC):
         """
         Returns the first node of the mainline after this node, or ``None`` if
         this node does not have any children.
+
+        Complexity is `O(1)`.
         """
         return self.variations[0] if self.variations else None
 
@@ -404,6 +432,8 @@ class GameNode(abc.ABC):
         """
         Parses the first valid ``[%eval ...]`` annotation in the comment of
         this node, if any.
+
+        Complexity is `O(n)`.
         """
         match = EVAL_REGEX.search(self.comment)
         if not match:
@@ -420,7 +450,7 @@ class GameNode(abc.ABC):
                 # who has been mated.
                 return chess.engine.PovScore(score, turn)
         else:
-            score = chess.engine.Cp(int(float(match.group("cp")) * 100))
+            score = chess.engine.Cp(round(float(match.group("cp")) * 100))
 
         return chess.engine.PovScore(score if turn else -score, turn)
 
@@ -428,6 +458,8 @@ class GameNode(abc.ABC):
         """
         Parses the first valid ``[%eval ...]`` annotation in the comment of
         this node and returns the corresponding depth, if any.
+
+        Complexity is `O(1)`.
         """
         match = EVAL_REGEX.search(self.comment)
         return int(match.group("depth")) if match and match.group("depth") else None
@@ -670,6 +702,8 @@ class ChildNode(GameNode):
         See :func:`chess.Board.san()`.
 
         Do not call this on the root node.
+
+        Complexity is `O(n)`.
         """
         return self.parent.board().san(self.move)
 
@@ -679,11 +713,17 @@ class ChildNode(GameNode):
         See :func:`chess.Board.uci()`.
 
         Do not call this on the root node.
+
+        Complexity is `O(n)`.
         """
         return self.parent.board().uci(self.move, chess960=chess960)
 
     def end(self) -> ChildNode:
-        """Follows the main variation to the end and returns the last node."""
+        """
+        Follows the main variation to the end and returns the last node.
+
+        Complexity is `O(n)`.
+        """
         return typing.cast(ChildNode, super().end())
 
     def _accept_node(self, parent_board: chess.Board, visitor: BaseVisitor[ResultT]) -> None:
@@ -810,11 +850,11 @@ class Game(GameNode):
             fen = setup.fen()
 
         if fen == type(setup).starting_fen:
-            self.headers.pop("SetUp", None)
             self.headers.pop("FEN", None)
+            self.headers.pop("SetUp", None)
         else:
-            self.headers["SetUp"] = "1"
             self.headers["FEN"] = fen
+            self.headers["SetUp"] = "1"
 
         if type(setup).aliases[0] == "Standard" and setup.chess960:
             self.headers["Variant"] = "Chess960"
@@ -872,12 +912,13 @@ class Game(GameNode):
         return GameBuilder(Game=cls)
 
     def __repr__(self) -> str:
-        return "<{} at {:#x} ({!r} vs. {!r}, {!r}{})>".format(
+        return "<{} at {:#x} ({!r} vs. {!r}, {!r} at {!r}{})>".format(
             type(self).__name__,
             id(self),
             self.headers.get("White", "?"),
             self.headers.get("Black", "?"),
             self.headers.get("Date", "????.??.??"),
+            self.headers.get("Site", "?"),
             f", {len(self.errors)} errors" if self.errors else "")
 
 
@@ -941,10 +982,7 @@ class Headers(MutableMapping[str, str]):
             self._others[key] = value
 
     def __getitem__(self, key: str) -> str:
-        if key in TAG_ROSTER:
-            return self._tag_roster[key]
-        else:
-            return self._others[key]
+        return self._tag_roster[key] if key in TAG_ROSTER else self._others[key]
 
     def __delitem__(self, key: str) -> None:
         if key in TAG_ROSTER:
@@ -957,7 +995,7 @@ class Headers(MutableMapping[str, str]):
             if key in self._tag_roster:
                 yield key
 
-        yield from sorted(self._others)
+        yield from self._others
 
     def __len__(self) -> int:
         return len(self._tag_roster) + len(self._others)
@@ -1042,6 +1080,13 @@ class BaseVisitor(abc.ABC, Generic[ResultT]):
         """Called after visiting game headers."""
         pass
 
+    def begin_parse_san(self, board: chess.Board, san: str) -> Optional[SkipType]:
+        """
+        When the visitor is used by a parser, this is called at the start of
+        each standard algebraic notation detailing a move.
+        """
+        pass
+
     def parse_san(self, board: chess.Board, san: str) -> chess.Move:
         """
         When the visitor is used by a parser, this is called to parse a move
@@ -1202,7 +1247,7 @@ class GameBuilder(BaseVisitor[GameT]):
         >>>
         >>> game = chess.pgn.read_game(pgn, Visitor=MyGameBuilder)
         """
-        LOGGER.exception("error during pgn parsing")
+        LOGGER.error("%s while parsing %r", error, self.game)
         self.game.errors.append(error)
 
     def result(self) -> GameT:
@@ -1681,14 +1726,15 @@ def read_game(handle: TextIO, *, Visitor: Any = GameBuilder) -> Any:
                 visitor.visit_result(token)
             else:
                 # Parse SAN tokens.
-                try:
-                    move = visitor.parse_san(board_stack[-1], token)
-                except ValueError as error:
-                    visitor.handle_error(error)
-                    skip_variation_depth = 1
-                else:
-                    visitor.visit_move(board_stack[-1], move)
-                    board_stack[-1].push(move)
+                if visitor.begin_parse_san(board_stack[-1], token) is not SKIP:
+                    try:
+                        move = visitor.parse_san(board_stack[-1], token)
+                    except ValueError as error:
+                        visitor.handle_error(error)
+                        skip_variation_depth = 1
+                    else:
+                        visitor.visit_move(board_stack[-1], move)
+                        board_stack[-1].push(move)
                 visitor.visit_board(board_stack[-1])
 
         if fresh_line:
