@@ -1,6 +1,6 @@
 # This file is part of the python-chess library.
 # Copyright (C) 2015 Jean-Noël Avila <jn.avila@free.fr>
-# Copyright (C) 2015-2020 Niklas Fiekas <niklas.fiekas@backscattering.de>
+# Copyright (C) 2015-2021 Niklas Fiekas <niklas.fiekas@backscattering.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,8 +15,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
 import ctypes
 import ctypes.util
+import dataclasses
 import fnmatch
 import logging
 import lzma
@@ -99,7 +102,7 @@ EGTB_MAXBLOCKSIZE = 65536
 
 
 def map24_b(s: int) -> int:
-    s = s - 8
+    s -= 8
     return ((s & 3) + s) >> 1
 
 def map88(x: int) -> int:
@@ -216,7 +219,7 @@ def init_ppp48_idx() -> Tuple[List[List[List[int]]], List[int], List[int], List[
                     ppp48_sq_x[idx] = i
                     ppp48_sq_y[idx] = j
                     ppp48_sq_z[idx] = k
-                    idx = idx + 1
+                    idx += 1
 
     return ppp48_idx, ppp48_sq_x, ppp48_sq_y, ppp48_sq_z
 
@@ -250,7 +253,7 @@ def init_aaa() -> Tuple[List[int], List[List[int]]]:
     accum = 0
     aaa_base = [0] * 64
     for a in range(64 - 1):
-        accum = accum + comb[a]
+        accum += comb[a]
         aaa_base[a + 1] = accum
 
     # Get aaa_xyz.
@@ -416,7 +419,7 @@ def init_kkidx() -> Tuple[List[List[int]], List[int], List[int]]:
 KKIDX, WKSQ, BKSQ = init_kkidx()
 
 
-def kxk_pctoindex(c: "Request") -> int:
+def kxk_pctoindex(c: Request) -> int:
     BLOCK_Ax = 64
 
     ft = flip_type(c.black_piece_squares[0], c.white_piece_squares[0])
@@ -443,7 +446,7 @@ def kxk_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_Ax + ws[1]
 
-def kapkb_pctoindex(c: "Request") -> int:
+def kapkb_pctoindex(c: Request) -> int:
     BLOCK_A = 64 * 64 * 64 * 64
     BLOCK_B = 64 * 64 * 64
     BLOCK_C = 64 * 64
@@ -473,7 +476,7 @@ def kapkb_pctoindex(c: "Request") -> int:
 
     return pslice * BLOCK_A + wk * BLOCK_B + bk * BLOCK_C + wa * BLOCK_D + ba
 
-def kabpk_pctoindex(c: "Request") -> int:
+def kabpk_pctoindex(c: Request) -> int:
     BLOCK_A = 64 * 64 * 64 * 64
     BLOCK_B = 64 * 64 * 64
     BLOCK_C = 64 * 64
@@ -497,7 +500,7 @@ def kabpk_pctoindex(c: "Request") -> int:
 
     return pslice * BLOCK_A + wk * BLOCK_B + bk * BLOCK_C + wa * BLOCK_D + wb
 
-def kabkp_pctoindex(c: "Request") -> int:
+def kabkp_pctoindex(c: Request) -> int:
     BLOCK_A = 64 * 64 * 64 * 64
     BLOCK_B = 64 * 64 * 64
     BLOCK_C = 64 * 64
@@ -526,7 +529,7 @@ def kabkp_pctoindex(c: "Request") -> int:
 
     return pslice * BLOCK_A + wk * BLOCK_B + bk * BLOCK_C + wa * BLOCK_D + wb
 
-def kaapk_pctoindex(c: "Request") -> int:
+def kaapk_pctoindex(c: Request) -> int:
     BLOCK_C = MAX_AAINDEX
     BLOCK_B = 64 * BLOCK_C
     BLOCK_A = 64 * BLOCK_B
@@ -554,7 +557,7 @@ def kaapk_pctoindex(c: "Request") -> int:
 
     return pslice * BLOCK_A + wk * BLOCK_B + bk * BLOCK_C + aa_combo
 
-def kaakp_pctoindex(c: "Request") -> int:
+def kaakp_pctoindex(c: Request) -> int:
     BLOCK_C = MAX_AAINDEX
     BLOCK_B = 64 * BLOCK_C
     BLOCK_A = 64 * BLOCK_B
@@ -583,7 +586,7 @@ def kaakp_pctoindex(c: "Request") -> int:
 
     return pslice * BLOCK_A + wk * BLOCK_B + bk * BLOCK_C + aa_combo
 
-def kapkp_pctoindex(c: "Request") -> int:
+def kapkp_pctoindex(c: Request) -> int:
     BLOCK_A = 64 * 64 * 64
     BLOCK_B = 64 * 64
     BLOCK_C = 64
@@ -614,7 +617,7 @@ def kapkp_pctoindex(c: "Request") -> int:
 
     return pp_slice * BLOCK_A + wk * BLOCK_B + bk * BLOCK_C + wa
 
-def kappk_pctoindex(c: "Request") -> int:
+def kappk_pctoindex(c: Request) -> int:
     BLOCK_A = 64 * 64 * 64
     BLOCK_B = 64 * 64
     BLOCK_C = 64
@@ -645,7 +648,7 @@ def kappk_pctoindex(c: "Request") -> int:
 
     return pp_slice * BLOCK_A + wk * BLOCK_B + bk * BLOCK_C + wa
 
-def kppka_pctoindex(c: "Request") -> int:
+def kppka_pctoindex(c: Request) -> int:
     BLOCK_A = 64 * 64 * 64
     BLOCK_B = 64 * 64
     BLOCK_C = 64
@@ -675,7 +678,7 @@ def kppka_pctoindex(c: "Request") -> int:
 
     return pp_slice * BLOCK_A + wk * BLOCK_B + bk * BLOCK_C + ba
 
-def kabck_pctoindex(c: "Request") -> int:
+def kabck_pctoindex(c: Request) -> int:
     N_WHITE = 4
     N_BLACK = 1
     BLOCK_A = 64 * 64 * 64
@@ -706,7 +709,7 @@ def kabck_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_A + ws[1] * BLOCK_B + ws[2] * BLOCK_C + ws[3]
 
-def kabbk_pctoindex(c: "Request") -> int:
+def kabbk_pctoindex(c: Request) -> int:
     N_WHITE = 4
     N_BLACK = 1
     BLOCK_Bx = 64
@@ -737,7 +740,7 @@ def kabbk_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_Ax + ai * BLOCK_Bx + ws[1]
 
-def kaabk_pctoindex(c: "Request") -> int:
+def kaabk_pctoindex(c: Request) -> int:
     N_WHITE = 4
     N_BLACK = 1
     BLOCK_Bx = 64
@@ -773,7 +776,7 @@ def aaa_getsubi(x: int, y: int, z: int) -> int:
     calc_idx = x + (y - 1) * y // 2 + bse
     return calc_idx
 
-def kaaak_pctoindex(c: "Request") -> int:
+def kaaak_pctoindex(c: Request) -> int:
     N_WHITE = 4
     N_BLACK = 1
     BLOCK_Ax = MAX_AAAINDEX
@@ -820,7 +823,7 @@ def kaaak_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_Ax + ai
 
-def kppkp_pctoindex(c: "Request") -> int:
+def kppkp_pctoindex(c: Request) -> int:
     BLOCK_Ax = MAX_PP48_INDEX * 64 * 64
     BLOCK_Bx = 64 * 64
     BLOCK_Cx = 64
@@ -851,7 +854,7 @@ def kppkp_pctoindex(c: "Request") -> int:
 
     return k * BLOCK_Ax + pp48_slice * BLOCK_Bx + wk * BLOCK_Cx + bk
 
-def kaakb_pctoindex(c: "Request") -> int:
+def kaakb_pctoindex(c: Request) -> int:
     N_WHITE = 3
     N_BLACK = 2
     BLOCK_Bx = 64
@@ -882,7 +885,7 @@ def kaakb_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_Ax + ai * BLOCK_Bx + bs[1]
 
-def kabkc_pctoindex(c: "Request") -> int:
+def kabkc_pctoindex(c: Request) -> int:
     N_WHITE = 3
     N_BLACK = 2
 
@@ -914,7 +917,7 @@ def kabkc_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_Ax + ws[1] * BLOCK_Bx + ws[2] * BLOCK_Cx + bs[1]
 
-def kpkp_pctoindex(c: "Request") -> int:
+def kpkp_pctoindex(c: Request) -> int:
     BLOCK_Ax = 64 * 64
     BLOCK_Bx = 64
 
@@ -942,7 +945,7 @@ def kpkp_pctoindex(c: "Request") -> int:
 
     return pp_slice * BLOCK_Ax + wk * BLOCK_Bx + bk
 
-def kppk_pctoindex(c: "Request") -> int:
+def kppk_pctoindex(c: Request) -> int:
     BLOCK_Ax = 64 * 64
     BLOCK_Bx = 64
     wk = c.white_piece_squares[0]
@@ -968,7 +971,7 @@ def kppk_pctoindex(c: "Request") -> int:
 
     return pp_slice * BLOCK_Ax + wk * BLOCK_Bx + bk
 
-def kapk_pctoindex(c: "Request") -> int:
+def kapk_pctoindex(c: Request) -> int:
     BLOCK_Ax = 64 * 64 * 64
     BLOCK_Bx = 64 * 64
     BLOCK_Cx = 64
@@ -994,7 +997,7 @@ def kapk_pctoindex(c: "Request") -> int:
 
     return pslice * BLOCK_Ax + wk * BLOCK_Bx + bk * BLOCK_Cx + wa
 
-def kabk_pctoindex(c: "Request") -> int:
+def kabk_pctoindex(c: Request) -> int:
     BLOCK_Ax = 64 * 64
     BLOCK_Bx = 64
 
@@ -1022,7 +1025,7 @@ def kabk_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_Ax + ws[1] * BLOCK_Bx + ws[2]
 
-def kakp_pctoindex(c: "Request") -> int:
+def kakp_pctoindex(c: Request) -> int:
     BLOCK_Ax = 64 * 64 * 64
     BLOCK_Bx = 64 * 64
     BLOCK_Cx = 64
@@ -1047,7 +1050,7 @@ def kakp_pctoindex(c: "Request") -> int:
 
     return pslice * BLOCK_Ax + wk * BLOCK_Bx + bk * BLOCK_Cx + wa
 
-def kaak_pctoindex(c: "Request") -> int:
+def kaak_pctoindex(c: Request) -> int:
     N_WHITE = 3
     N_BLACK = 1
     BLOCK_Ax = MAX_AAINDEX
@@ -1077,7 +1080,7 @@ def kaak_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_Ax + ai
 
-def kakb_pctoindex(c: "Request") -> int:
+def kakb_pctoindex(c: Request) -> int:
     BLOCK_Ax = 64 * 64
     BLOCK_Bx = 64
 
@@ -1111,7 +1114,7 @@ def kakb_pctoindex(c: "Request") -> int:
 
     return ki * BLOCK_Ax + ws[1] * BLOCK_Bx + bs[1]
 
-def kpk_pctoindex(c: "Request") -> int:
+def kpk_pctoindex(c: Request) -> int:
     BLOCK_A = 64 * 64
     BLOCK_B = 64
 
@@ -1135,7 +1138,7 @@ def kpk_pctoindex(c: "Request") -> int:
     res = pslice * BLOCK_A + wk * BLOCK_B + bk
     return res
 
-def kpppk_pctoindex(c: "Request") -> int:
+def kpppk_pctoindex(c: Request) -> int:
     BLOCK_A = 64 * 64
     BLOCK_B = 64
 
@@ -1171,178 +1174,178 @@ def kpppk_pctoindex(c: "Request") -> int:
     return ppp48_slice * BLOCK_A + wk * BLOCK_B + bk
 
 
-class Endgamekey:
-    def __init__(self, maxindex: int, slice_n: int, pctoi: Callable[["Request"], int]) -> None:
+class EndgameKey:
+    def __init__(self, maxindex: int, slice_n: int, pctoi: Callable[[Request], int]):
         self.maxindex = maxindex
         self.slice_n = slice_n
         self.pctoi = pctoi
 
 EGKEY = {
-    "kqk": Endgamekey(MAX_KXK, 1, kxk_pctoindex),
-    "krk": Endgamekey(MAX_KXK, 1, kxk_pctoindex),
-    "kbk": Endgamekey(MAX_KXK, 1, kxk_pctoindex),
-    "knk": Endgamekey(MAX_KXK, 1, kxk_pctoindex),
-    "kpk": Endgamekey(MAX_kpk, 24, kpk_pctoindex),
+    "kqk": EndgameKey(MAX_KXK, 1, kxk_pctoindex),
+    "krk": EndgameKey(MAX_KXK, 1, kxk_pctoindex),
+    "kbk": EndgameKey(MAX_KXK, 1, kxk_pctoindex),
+    "knk": EndgameKey(MAX_KXK, 1, kxk_pctoindex),
+    "kpk": EndgameKey(MAX_kpk, 24, kpk_pctoindex),
 
-    "kqkq": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
-    "kqkr": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
-    "kqkb": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
-    "kqkn": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
+    "kqkq": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
+    "kqkr": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
+    "kqkb": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
+    "kqkn": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
 
-    "krkr": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
-    "krkb": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
-    "krkn": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
+    "krkr": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
+    "krkb": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
+    "krkn": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
 
-    "kbkb": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
-    "kbkn": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
+    "kbkb": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
+    "kbkn": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
 
-    "knkn": Endgamekey(MAX_kakb, 1, kakb_pctoindex),
+    "knkn": EndgameKey(MAX_kakb, 1, kakb_pctoindex),
 
-    "kqqk": Endgamekey(MAX_kaak, 1, kaak_pctoindex),
-    "kqrk": Endgamekey(MAX_kabk, 1, kabk_pctoindex),
-    "kqbk": Endgamekey(MAX_kabk, 1, kabk_pctoindex),
-    "kqnk": Endgamekey(MAX_kabk, 1, kabk_pctoindex),
+    "kqqk": EndgameKey(MAX_kaak, 1, kaak_pctoindex),
+    "kqrk": EndgameKey(MAX_kabk, 1, kabk_pctoindex),
+    "kqbk": EndgameKey(MAX_kabk, 1, kabk_pctoindex),
+    "kqnk": EndgameKey(MAX_kabk, 1, kabk_pctoindex),
 
-    "krrk": Endgamekey(MAX_kaak, 1, kaak_pctoindex),
-    "krbk": Endgamekey(MAX_kabk, 1, kabk_pctoindex),
-    "krnk": Endgamekey(MAX_kabk, 1, kabk_pctoindex),
+    "krrk": EndgameKey(MAX_kaak, 1, kaak_pctoindex),
+    "krbk": EndgameKey(MAX_kabk, 1, kabk_pctoindex),
+    "krnk": EndgameKey(MAX_kabk, 1, kabk_pctoindex),
 
-    "kbbk": Endgamekey(MAX_kaak, 1, kaak_pctoindex),
-    "kbnk": Endgamekey(MAX_kabk, 1, kabk_pctoindex),
+    "kbbk": EndgameKey(MAX_kaak, 1, kaak_pctoindex),
+    "kbnk": EndgameKey(MAX_kabk, 1, kabk_pctoindex),
 
-    "knnk": Endgamekey(MAX_kaak, 1, kaak_pctoindex),
-    "kqkp": Endgamekey(MAX_kakp, 24, kakp_pctoindex),
-    "krkp": Endgamekey(MAX_kakp, 24, kakp_pctoindex),
-    "kbkp": Endgamekey(MAX_kakp, 24, kakp_pctoindex),
-    "knkp": Endgamekey(MAX_kakp, 24, kakp_pctoindex),
+    "knnk": EndgameKey(MAX_kaak, 1, kaak_pctoindex),
+    "kqkp": EndgameKey(MAX_kakp, 24, kakp_pctoindex),
+    "krkp": EndgameKey(MAX_kakp, 24, kakp_pctoindex),
+    "kbkp": EndgameKey(MAX_kakp, 24, kakp_pctoindex),
+    "knkp": EndgameKey(MAX_kakp, 24, kakp_pctoindex),
 
-    "kqpk": Endgamekey(MAX_kapk, 24, kapk_pctoindex),
-    "krpk": Endgamekey(MAX_kapk, 24, kapk_pctoindex),
-    "kbpk": Endgamekey(MAX_kapk, 24, kapk_pctoindex),
-    "knpk": Endgamekey(MAX_kapk, 24, kapk_pctoindex),
+    "kqpk": EndgameKey(MAX_kapk, 24, kapk_pctoindex),
+    "krpk": EndgameKey(MAX_kapk, 24, kapk_pctoindex),
+    "kbpk": EndgameKey(MAX_kapk, 24, kapk_pctoindex),
+    "knpk": EndgameKey(MAX_kapk, 24, kapk_pctoindex),
 
-    "kppk": Endgamekey(MAX_kppk, MAX_PPINDEX, kppk_pctoindex),
+    "kppk": EndgameKey(MAX_kppk, MAX_PPINDEX, kppk_pctoindex),
 
-    "kpkp": Endgamekey(MAX_kpkp, MAX_PpINDEX, kpkp_pctoindex),
+    "kpkp": EndgameKey(MAX_kpkp, MAX_PpINDEX, kpkp_pctoindex),
 
-    "kppkp": Endgamekey(MAX_kppkp, 24 * MAX_PP48_INDEX, kppkp_pctoindex),
+    "kppkp": EndgameKey(MAX_kppkp, 24 * MAX_PP48_INDEX, kppkp_pctoindex),
 
-    "kbbkr": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "kbbkb": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "knnkb": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "knnkn": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kbbkr": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kbbkb": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "knnkb": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "knnkn": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
 
-    "kqqqk": Endgamekey(MAX_kaaak, 1, kaaak_pctoindex),
-    "kqqrk": Endgamekey(MAX_kaabk, 1, kaabk_pctoindex),
-    "kqqbk": Endgamekey(MAX_kaabk, 1, kaabk_pctoindex),
-    "kqqnk": Endgamekey(MAX_kaabk, 1, kaabk_pctoindex),
-    "kqrrk": Endgamekey(MAX_kabbk, 1, kabbk_pctoindex),
-    "kqrbk": Endgamekey(MAX_kabck, 1, kabck_pctoindex),
-    "kqrnk": Endgamekey(MAX_kabck, 1, kabck_pctoindex),
-    "kqbbk": Endgamekey(MAX_kabbk, 1, kabbk_pctoindex),
-    "kqbnk": Endgamekey(MAX_kabck, 1, kabck_pctoindex),
-    "kqnnk": Endgamekey(MAX_kabbk, 1, kabbk_pctoindex),
-    "krrrk": Endgamekey(MAX_kaaak, 1, kaaak_pctoindex),
-    "krrbk": Endgamekey(MAX_kaabk, 1, kaabk_pctoindex),
-    "krrnk": Endgamekey(MAX_kaabk, 1, kaabk_pctoindex),
-    "krbbk": Endgamekey(MAX_kabbk, 1, kabbk_pctoindex),
-    "krbnk": Endgamekey(MAX_kabck, 1, kabck_pctoindex),
-    "krnnk": Endgamekey(MAX_kabbk, 1, kabbk_pctoindex),
-    "kbbbk": Endgamekey(MAX_kaaak, 1, kaaak_pctoindex),
-    "kbbnk": Endgamekey(MAX_kaabk, 1, kaabk_pctoindex),
-    "kbnnk": Endgamekey(MAX_kabbk, 1, kabbk_pctoindex),
-    "knnnk": Endgamekey(MAX_kaaak, 1, kaaak_pctoindex),
-    "kqqkq": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "kqqkr": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "kqqkb": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "kqqkn": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "kqrkq": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqrkr": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqrkb": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqrkn": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqbkq": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqbkr": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqbkb": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqbkn": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqnkq": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqnkr": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqnkb": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kqnkn": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "krrkq": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "krrkr": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "krrkb": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "krrkn": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "krbkq": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "krbkr": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "krbkb": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "krbkn": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "krnkq": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "krnkr": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "krnkb": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "krnkn": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kbbkq": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "kbbkn": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "kbnkq": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kbnkr": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kbnkb": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "kbnkn": Endgamekey(MAX_kabkc, 1, kabkc_pctoindex),
-    "knnkq": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
-    "knnkr": Endgamekey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kqqqk": EndgameKey(MAX_kaaak, 1, kaaak_pctoindex),
+    "kqqrk": EndgameKey(MAX_kaabk, 1, kaabk_pctoindex),
+    "kqqbk": EndgameKey(MAX_kaabk, 1, kaabk_pctoindex),
+    "kqqnk": EndgameKey(MAX_kaabk, 1, kaabk_pctoindex),
+    "kqrrk": EndgameKey(MAX_kabbk, 1, kabbk_pctoindex),
+    "kqrbk": EndgameKey(MAX_kabck, 1, kabck_pctoindex),
+    "kqrnk": EndgameKey(MAX_kabck, 1, kabck_pctoindex),
+    "kqbbk": EndgameKey(MAX_kabbk, 1, kabbk_pctoindex),
+    "kqbnk": EndgameKey(MAX_kabck, 1, kabck_pctoindex),
+    "kqnnk": EndgameKey(MAX_kabbk, 1, kabbk_pctoindex),
+    "krrrk": EndgameKey(MAX_kaaak, 1, kaaak_pctoindex),
+    "krrbk": EndgameKey(MAX_kaabk, 1, kaabk_pctoindex),
+    "krrnk": EndgameKey(MAX_kaabk, 1, kaabk_pctoindex),
+    "krbbk": EndgameKey(MAX_kabbk, 1, kabbk_pctoindex),
+    "krbnk": EndgameKey(MAX_kabck, 1, kabck_pctoindex),
+    "krnnk": EndgameKey(MAX_kabbk, 1, kabbk_pctoindex),
+    "kbbbk": EndgameKey(MAX_kaaak, 1, kaaak_pctoindex),
+    "kbbnk": EndgameKey(MAX_kaabk, 1, kaabk_pctoindex),
+    "kbnnk": EndgameKey(MAX_kabbk, 1, kabbk_pctoindex),
+    "knnnk": EndgameKey(MAX_kaaak, 1, kaaak_pctoindex),
+    "kqqkq": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kqqkr": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kqqkb": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kqqkn": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kqrkq": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqrkr": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqrkb": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqrkn": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqbkq": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqbkr": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqbkb": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqbkn": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqnkq": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqnkr": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqnkb": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kqnkn": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "krrkq": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "krrkr": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "krrkb": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "krrkn": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "krbkq": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "krbkr": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "krbkb": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "krbkn": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "krnkq": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "krnkr": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "krnkb": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "krnkn": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kbbkq": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kbbkn": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "kbnkq": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kbnkr": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kbnkb": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "kbnkn": EndgameKey(MAX_kabkc, 1, kabkc_pctoindex),
+    "knnkq": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
+    "knnkr": EndgameKey(MAX_kaakb, 1, kaakb_pctoindex),
 
-    "kqqpk": Endgamekey(MAX_kaapk, 24, kaapk_pctoindex),
-    "kqrpk": Endgamekey(MAX_kabpk, 24, kabpk_pctoindex),
-    "kqbpk": Endgamekey(MAX_kabpk, 24, kabpk_pctoindex),
-    "kqnpk": Endgamekey(MAX_kabpk, 24, kabpk_pctoindex),
-    "krrpk": Endgamekey(MAX_kaapk, 24, kaapk_pctoindex),
-    "krbpk": Endgamekey(MAX_kabpk, 24, kabpk_pctoindex),
-    "krnpk": Endgamekey(MAX_kabpk, 24, kabpk_pctoindex),
-    "kbbpk": Endgamekey(MAX_kaapk, 24, kaapk_pctoindex),
-    "kbnpk": Endgamekey(MAX_kabpk, 24, kabpk_pctoindex),
-    "knnpk": Endgamekey(MAX_kaapk, 24, kaapk_pctoindex),
+    "kqqpk": EndgameKey(MAX_kaapk, 24, kaapk_pctoindex),
+    "kqrpk": EndgameKey(MAX_kabpk, 24, kabpk_pctoindex),
+    "kqbpk": EndgameKey(MAX_kabpk, 24, kabpk_pctoindex),
+    "kqnpk": EndgameKey(MAX_kabpk, 24, kabpk_pctoindex),
+    "krrpk": EndgameKey(MAX_kaapk, 24, kaapk_pctoindex),
+    "krbpk": EndgameKey(MAX_kabpk, 24, kabpk_pctoindex),
+    "krnpk": EndgameKey(MAX_kabpk, 24, kabpk_pctoindex),
+    "kbbpk": EndgameKey(MAX_kaapk, 24, kaapk_pctoindex),
+    "kbnpk": EndgameKey(MAX_kabpk, 24, kabpk_pctoindex),
+    "knnpk": EndgameKey(MAX_kaapk, 24, kaapk_pctoindex),
 
-    "kqppk": Endgamekey(MAX_kappk, MAX_PPINDEX, kappk_pctoindex),
-    "krppk": Endgamekey(MAX_kappk, MAX_PPINDEX, kappk_pctoindex),
-    "kbppk": Endgamekey(MAX_kappk, MAX_PPINDEX, kappk_pctoindex),
-    "knppk": Endgamekey(MAX_kappk, MAX_PPINDEX, kappk_pctoindex),
+    "kqppk": EndgameKey(MAX_kappk, MAX_PPINDEX, kappk_pctoindex),
+    "krppk": EndgameKey(MAX_kappk, MAX_PPINDEX, kappk_pctoindex),
+    "kbppk": EndgameKey(MAX_kappk, MAX_PPINDEX, kappk_pctoindex),
+    "knppk": EndgameKey(MAX_kappk, MAX_PPINDEX, kappk_pctoindex),
 
-    "kqpkq": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "kqpkr": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "kqpkb": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "kqpkn": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "krpkq": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "krpkr": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "krpkb": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "krpkn": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "kbpkq": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "kbpkr": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "kbpkb": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "kbpkn": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "knpkq": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "knpkr": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "knpkb": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "knpkn": Endgamekey(MAX_kapkb, 24, kapkb_pctoindex),
-    "kppkq": Endgamekey(MAX_kppka, MAX_PPINDEX, kppka_pctoindex),
-    "kppkr": Endgamekey(MAX_kppka, MAX_PPINDEX, kppka_pctoindex),
-    "kppkb": Endgamekey(MAX_kppka, MAX_PPINDEX, kppka_pctoindex),
-    "kppkn": Endgamekey(MAX_kppka, MAX_PPINDEX, kppka_pctoindex),
+    "kqpkq": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "kqpkr": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "kqpkb": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "kqpkn": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "krpkq": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "krpkr": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "krpkb": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "krpkn": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "kbpkq": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "kbpkr": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "kbpkb": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "kbpkn": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "knpkq": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "knpkr": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "knpkb": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "knpkn": EndgameKey(MAX_kapkb, 24, kapkb_pctoindex),
+    "kppkq": EndgameKey(MAX_kppka, MAX_PPINDEX, kppka_pctoindex),
+    "kppkr": EndgameKey(MAX_kppka, MAX_PPINDEX, kppka_pctoindex),
+    "kppkb": EndgameKey(MAX_kppka, MAX_PPINDEX, kppka_pctoindex),
+    "kppkn": EndgameKey(MAX_kppka, MAX_PPINDEX, kppka_pctoindex),
 
-    "kqqkp": Endgamekey(MAX_kaakp, 24, kaakp_pctoindex),
-    "kqrkp": Endgamekey(MAX_kabkp, 24, kabkp_pctoindex),
-    "kqbkp": Endgamekey(MAX_kabkp, 24, kabkp_pctoindex),
-    "kqnkp": Endgamekey(MAX_kabkp, 24, kabkp_pctoindex),
-    "krrkp": Endgamekey(MAX_kaakp, 24, kaakp_pctoindex),
-    "krbkp": Endgamekey(MAX_kabkp, 24, kabkp_pctoindex),
-    "krnkp": Endgamekey(MAX_kabkp, 24, kabkp_pctoindex),
-    "kbbkp": Endgamekey(MAX_kaakp, 24, kaakp_pctoindex),
-    "kbnkp": Endgamekey(MAX_kabkp, 24, kabkp_pctoindex),
-    "knnkp": Endgamekey(MAX_kaakp, 24, kaakp_pctoindex),
+    "kqqkp": EndgameKey(MAX_kaakp, 24, kaakp_pctoindex),
+    "kqrkp": EndgameKey(MAX_kabkp, 24, kabkp_pctoindex),
+    "kqbkp": EndgameKey(MAX_kabkp, 24, kabkp_pctoindex),
+    "kqnkp": EndgameKey(MAX_kabkp, 24, kabkp_pctoindex),
+    "krrkp": EndgameKey(MAX_kaakp, 24, kaakp_pctoindex),
+    "krbkp": EndgameKey(MAX_kabkp, 24, kabkp_pctoindex),
+    "krnkp": EndgameKey(MAX_kabkp, 24, kabkp_pctoindex),
+    "kbbkp": EndgameKey(MAX_kaakp, 24, kaakp_pctoindex),
+    "kbnkp": EndgameKey(MAX_kabkp, 24, kabkp_pctoindex),
+    "knnkp": EndgameKey(MAX_kaakp, 24, kaakp_pctoindex),
 
-    "kqpkp": Endgamekey(MAX_kapkp, MAX_PpINDEX, kapkp_pctoindex),
-    "krpkp": Endgamekey(MAX_kapkp, MAX_PpINDEX, kapkp_pctoindex),
-    "kbpkp": Endgamekey(MAX_kapkp, MAX_PpINDEX, kapkp_pctoindex),
-    "knpkp": Endgamekey(MAX_kapkp, MAX_PpINDEX, kapkp_pctoindex),
+    "kqpkp": EndgameKey(MAX_kapkp, MAX_PpINDEX, kapkp_pctoindex),
+    "krpkp": EndgameKey(MAX_kapkp, MAX_PpINDEX, kapkp_pctoindex),
+    "kbpkp": EndgameKey(MAX_kapkp, MAX_PpINDEX, kapkp_pctoindex),
+    "knpkp": EndgameKey(MAX_kapkp, MAX_PpINDEX, kapkp_pctoindex),
 
-    "kpppk": Endgamekey(MAX_kpppk, MAX_PPP48_INDEX, kpppk_pctoindex),
+    "kpppk": EndgameKey(MAX_kpppk, MAX_PPP48_INDEX, kpppk_pctoindex),
 }
 
 
@@ -1517,11 +1520,11 @@ class Request:
         self.epsq = epsq
 
 
-class Zipinfo:
-    def __init__(self, extraoffset: int, totalblocks: int, blockindex: List[int]) -> None:
-        self.extraoffset = extraoffset
-        self.totalblocks = totalblocks
-        self.blockindex = blockindex
+@dataclasses.dataclass
+class ZipInfo:
+    extraoffset: int
+    totalblocks: int
+    blockindex: List[int]
 
 
 class PythonTablebase:
@@ -1531,7 +1534,7 @@ class PythonTablebase:
         self.available_tables: Dict[str, str] = {}
 
         self.streams: Dict[str, BinaryIO] = {}
-        self.zipinfo: Dict[str, Zipinfo] = {}
+        self.zipinfo: Dict[str, ZipInfo] = {}
 
         self.block_cache: Dict[Tuple[str, int, int], TableBlock] = {}
         self.block_age = 0
@@ -1868,7 +1871,7 @@ class PythonTablebase:
 
         return dtm
 
-    def egtb_loadindexes(self, egkey: str, stream: BinaryIO) -> Zipinfo:
+    def egtb_loadindexes(self, egkey: str, stream: BinaryIO) -> ZipInfo:
         zipinfo = self.zipinfo.get(egkey)
 
         if zipinfo is None:
@@ -1884,7 +1887,7 @@ class PythonTablebase:
             IndexStruct = struct.Struct("<" + "I" * n_idx)
             p = list(IndexStruct.unpack(stream.read(IndexStruct.size)))
 
-            zipinfo = Zipinfo(extraoffset=0, totalblocks=n_idx, blockindex=p)
+            zipinfo = ZipInfo(extraoffset=0, totalblocks=n_idx, blockindex=p)
             self.zipinfo[egkey] = zipinfo
 
         return zipinfo
@@ -1900,7 +1903,7 @@ class PythonTablebase:
         stream.seek(i)
         return i
 
-    def __enter__(self) -> "PythonTablebase":
+    def __enter__(self) -> PythonTablebase:
         return self
 
     def __exit__(self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException], traceback: Optional[TracebackType]) -> None:
@@ -1913,7 +1916,7 @@ class NativeTablebase:
     Has the same interface as :class:`~chess.gaviota.PythonTablebase`.
     """
 
-    def __init__(self, libgtb) -> None:
+    def __init__(self, libgtb: ctypes.CDLL) -> None:
         self.paths: List[str] = []
 
         self.libgtb = libgtb
@@ -2063,14 +2066,14 @@ class NativeTablebase:
             self.libgtb.tbcache_done()
             self.libgtb.tb_done()
 
-    def __enter__(self) -> "NativeTablebase":
+    def __enter__(self) -> NativeTablebase:
         return self
 
     def __exit__(self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException], traceback: Optional[TracebackType]) -> None:
         self.close()
 
 
-def open_tablebase_native(directory: str, *, libgtb=None, LibraryLoader=ctypes.cdll) -> NativeTablebase:
+def open_tablebase_native(directory: str, *, libgtb: Optional[str] = None, LibraryLoader: ctypes.LibraryLoader[ctypes.CDLL] = ctypes.cdll) -> NativeTablebase:
     """
     Opens a collection of tables for probing using libgtb.
 
@@ -2086,7 +2089,7 @@ def open_tablebase_native(directory: str, *, libgtb=None, LibraryLoader=ctypes.c
     return tables
 
 
-def open_tablebase(directory: str, *, libgtb=None, LibraryLoader=ctypes.cdll) -> Union[NativeTablebase, PythonTablebase]:
+def open_tablebase(directory: str, *, libgtb: Optional[str] = None, LibraryLoader: ctypes.LibraryLoader[ctypes.CDLL] = ctypes.cdll) -> Union[NativeTablebase, PythonTablebase]:
     """
     Opens a collection of tables for probing.
 
