@@ -85,16 +85,25 @@ class SuicideBoard(chess.Board):
             return self.is_stalemate() and self._material_balance() == 0
 
     def has_insufficient_material(self, color: chess.Color) -> bool:
-        if self.occupied != self.bishops:
+        if not self.occupied_co[color]:
+            return False
+        elif not self.occupied_co[not color]:
+            return True
+        elif self.occupied == self.bishops:
+            # In a position with only bishops, check if all our bishops can be
+            # captured.
+            we_some_on_light = bool(self.occupied_co[color] & chess.BB_LIGHT_SQUARES)
+            we_some_on_dark = bool(self.occupied_co[color] & chess.BB_DARK_SQUARES)
+            they_all_on_dark = not (self.occupied_co[not color] & chess.BB_LIGHT_SQUARES)
+            they_all_on_light = not (self.occupied_co[not color] & chess.BB_DARK_SQUARES)
+            return (we_some_on_light and they_all_on_dark) or (we_some_on_dark and they_all_on_light)
+        elif self.occupied == self.knights and chess.popcount(self.knights) == 2:
+            return (
+                self.turn == color ^
+                bool(self.occupied_co[chess.WHITE] & chess.BB_LIGHT_SQUARES) ^
+                bool(self.occupied_co[chess.BLACK] & chess.BB_DARK_SQUARES))
+        else:
             return False
-
-        # In a position with only bishops, check if all our bishops can be
-        # captured.
-        we_some_on_light = bool(self.occupied_co[color] & chess.BB_LIGHT_SQUARES)
-        we_some_on_dark = bool(self.occupied_co[color] & chess.BB_DARK_SQUARES)
-        they_all_on_dark = not (self.occupied_co[not color] & chess.BB_LIGHT_SQUARES)
-        they_all_on_light = not (self.occupied_co[not color] & chess.BB_DARK_SQUARES)
-        return (we_some_on_light and they_all_on_dark) or (we_some_on_dark and they_all_on_light)
 
     def generate_pseudo_legal_moves(self, from_mask: chess.Bitboard = chess.BB_ALL, to_mask: chess.Bitboard = chess.BB_ALL) -> Iterator[chess.Move]:
         for move in super().generate_pseudo_legal_moves(from_mask, to_mask):
@@ -201,7 +210,6 @@ class AtomicBoard(chess.Board):
     tbw_magic = b"\x55\x8d\xa4\x49"
     tbz_magic = b"\x91\xa9\x5e\xeb"
     connected_kings = True
-    one_king = True
 
     def is_variant_end(self) -> bool:
         return not all(self.kings & side for side in self.occupied_co)
@@ -980,7 +988,7 @@ class CrazyhouseBoard(chess.Board):
                 uci = "P" + uci
             move = chess.Move.from_uci(uci)
             if not self.is_legal(move):
-                raise ValueError(f"illegal drop san: {san!r} in {self.fen()}")
+                raise chess.IllegalMoveError(f"illegal drop san: {san!r} in {self.fen()}")
             return move
         else:
             return super().parse_san(san)
