@@ -559,7 +559,6 @@ class Table:
 
         self.write_lock = threading.RLock()
         self.initialized = False
-        self.fd: Optional[int] = None
         self.data: Optional[mmap.mmap] = None
 
         self.read_condition = threading.Condition()
@@ -602,23 +601,23 @@ class Table:
                         self.enc_type = 1 + j
 
     def init_mmap(self) -> None:
-        with self.write_lock:
-            # Open fd.
-            if self.fd is None:
-                self.fd = os.open(self.path, os.O_RDONLY | os.O_BINARY if hasattr(os, "O_BINARY") else os.O_RDONLY)
+        if self.data is None:
+            fd = os.open(self.path, os.O_RDONLY | os.O_BINARY if hasattr(os, "O_BINARY") else os.O_RDONLY)
+            try:
+                data = mmap.mmap(fd, 0, access=mmap.ACCESS_READ)
+            finally:
+                os.close(fd)
 
-            # Open mmap.
-            if self.data is None:
-                data = mmap.mmap(self.fd, 0, access=mmap.ACCESS_READ)
-                if data.size() % 64 != 16:
-                    raise IOError(f"invalid file size: ensure {self.path!r} is a valid syzygy tablebase file")
-                self.data = data
+            if data.size() % 64 != 16:
+                raise IOError(f"invalid file size: ensure {self.path!r} is a valid syzygy tablebase file")
 
-                try:
-                    # Python 3.8
-                    self.data.madvise(mmap.MADV_RANDOM)
-                except AttributeError:
-                    pass
+            try:
+                # Python 3.8
+                data.madvise(mmap.MADV_RANDOM)
+            except AttributeError:
+                pass
+
+            self.data = data
 
     def check_magic(self, magic: Optional[bytes], pawnless_magic: Optional[bytes]) -> None:
         assert self.data
@@ -1045,10 +1044,6 @@ class Table:
                     self.data.close()
                     self.data = None
 
-                if self.fd is not None:
-                    os.close(self.fd)
-                    self.fd = None
-
     def __enter__(self: TableT) -> TableT:
         return self
 
@@ -1487,10 +1482,6 @@ class DtzTable(Table):
 class Tablebase:
     """
     Manages a collection of tablebase files for probing.
-
-    If *max_fds* is not ``None``, will at most use *max_fds* open file
-    descriptors at any given time. The least recently used tables are closed,
-    if necessary.
     """
     def __init__(self, *, max_fds: Optional[int] = 128, VariantBoard: Type[chess.Board] = chess.Board) -> None:
         self.variant = VariantBoard
@@ -1986,6 +1977,10 @@ def open_tablebase(directory: str, *, load_wdl: bool = True, load_dtz: bool = Tr
         are often distributed separately, but are both required for 6-piece
         positions. Use :func:`~chess.syzygy.Tablebase.add_directory()` to load
         tables from additional directories.
+
+    :param max_fds: If *max_fds* is not ``None``, will at most use *max_fds*
+        open file descriptors at any given time. The least recently used tables
+        are closed, if necessary.
     """
     tables = Tablebase(max_fds=max_fds, VariantBoard=VariantBoard)
     tables.add_directory(directory, load_wdl=load_wdl, load_dtz=load_dtz)
